var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var cmdm = require('../lib/tfcommand');
var buildifm = require('vso-node-api/interfaces/BuildInterfaces');
var argm = require('../lib/arguments');
var os = require('os');
var trace = require('../lib/trace');
function describe() {
    return 'queue a build';
}
exports.describe = describe;
function getCommand() {
    // this just offers description for help and to offer sub commands
    return new BuildQueue();
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var BuildQueue = (function (_super) {
    __extends(BuildQueue, _super);
    function BuildQueue() {
        _super.apply(this, arguments);
        this.requiredArguments = [argm.PROJECT_NAME];
        this.optionalArguments = [argm.DEFINITION_ID, argm.DEFINITION_NAME];
    }
    BuildQueue.prototype.getArguments = function () {
        return _super.prototype.getArguments.call(this)
            + os.EOL + '\tmust supply ' + argm.DEFINITION_ID + ' or ' + argm.DEFINITION_NAME;
    };
    BuildQueue.prototype.exec = function (args, options) {
        var _this = this;
        trace('build-queue.exec');
        trace('Initializing Build API...');
        var buildapi = this.getWebApi().getQBuildApi();
        return this.checkArguments(args, options).then(function (allArguments) {
            var project = allArguments[argm.PROJECT_NAME.name];
            var definitionName = allArguments[argm.DEFINITION_NAME.name];
            var definitionId = allArguments[argm.DEFINITION_ID.name];
            if (definitionId) {
                trace('Searching for definitions with id ' + definitionId);
                return buildapi.getDefinition(definitionId, project).then(function (definition) {
                    return _this._queueBuild(buildapi, definition, project);
                });
            }
            else if (definitionName) {
                trace('No definition id provided, Searching for definitions with name: ' + definitionName);
                return buildapi.getDefinitions(project, definitionName).then(function (definitions) {
                    if (definitions.length > 0) {
                        var definition = definitions[0];
                        return _this._queueBuild(buildapi, definition, project);
                    }
                    else {
                        trace('No definition found with name ' + definitionName);
                        throw new Error('No definition found with name ' + definitionName);
                    }
                });
            }
            else {
                trace('neither definitionId nor definitionName provided.');
                throw new Error('definitionId or definitionName required. Try adding a switch to the end of your command --definitionId <definitionId>');
            }
        });
    };
    BuildQueue.prototype.output = function (data) {
        if (!data) {
            throw new Error('no build supplied');
        }
        var build = data;
        console.log();
        console.log('Queued new build:');
        console.log('id   : ' + build.id);
        console.log('definition name: ' + build.definition.name);
        console.log('requested by : ' + build.requestedBy.displayName);
        console.log('status : ' + buildifm.BuildStatus[build.status]);
        console.log('queue time : ' + build.queueTime);
    };
    BuildQueue.prototype._queueBuild = function (buildapi, definition, project) {
        trace('Queueing build...');
        var build = {
            definition: definition
        };
        return buildapi.queueBuild(build, project);
    };
    return BuildQueue;
})(cmdm.TfCommand);
exports.BuildQueue = BuildQueue;
