/// <reference path="../../definitions/q/Q.d.ts"/>
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var cmdm = require('../lib/tfcommand');
var argm = require('../lib/arguments');
var Q = require('q');
var trace = require('../lib/trace');
function describe() {
    return 'delete a build task';
}
exports.describe = describe;
function getCommand() {
    return new BuildTaskDelete();
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var BuildTaskDelete = (function (_super) {
    __extends(BuildTaskDelete, _super);
    function BuildTaskDelete() {
        _super.apply(this, arguments);
        this.requiredArguments = [argm.TASK_ID];
    }
    BuildTaskDelete.prototype.exec = function (args, options) {
        var _this = this;
        trace("build-tasks-delete.exec");
        var defer = Q.defer();
        this.checkArguments(args, options).then(function (allArguments) {
            var taskId = allArguments[argm.TASK_ID.name];
            trace("Initializing agent API...");
            var agentapi = _this.getWebApi().getTaskAgentApi(_this.connection.accountUrl);
            trace("Searching for tasks with id: " + taskId);
            agentapi.getTaskContent(taskId, "", function (err, statusCode, tasks) {
                trace("Found " + tasks.length + " tasks with provided id.");
                if (tasks.length > 0) {
                    trace("Deleting task(s)...");
                    agentapi.deleteTaskDefinition(taskId, function (err, statusCode) {
                        if (err) {
                            trace("Call to TaskAgentApi.deleteTaskDefinition failed with code " + statusCode + ". Message: " + err.message);
                            err.statusCode = statusCode;
                            defer.reject(err);
                        }
                        else {
                            trace("Success.");
                            defer.resolve({
                                id: taskId
                            });
                        }
                    });
                }
                else {
                    trace("No task.");
                    defer.reject(new Error("No task found with provided ID: " + taskId));
                }
            });
        })
            .fail(function (err) {
            trace('Failed to gather inputs. Message: ' + err.message);
            defer.reject(err);
        });
        return defer.promise;
    };
    BuildTaskDelete.prototype.output = function (data) {
        console.log('task: ' + data.id + ' deleted successfully!');
    };
    return BuildTaskDelete;
})(cmdm.TfCommand);
exports.BuildTaskDelete = BuildTaskDelete;
