/// <reference path="../../definitions/q/Q.d.ts"/>
/// <reference path="../../definitions/node/node.d.ts"/>
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var cmdm = require('../lib/tfcommand');
var vm = require('../lib/jsonvalidate');
var Q = require('q');
var path = require('path');
var argm = require('../lib/arguments');
var archiver = require('archiver');
var trace = require('../lib/trace');
function describe() {
    return 'upload a build task';
}
exports.describe = describe;
function getCommand() {
    return new BuildTaskUpload();
}
exports.getCommand = getCommand;
// requires auth, connect etc...
exports.isServerOperation = true;
// unless you have a good reason, should not hide
exports.hideBanner = false;
var c_taskJsonFile = 'task.json';
var BuildTaskUpload = (function (_super) {
    __extends(BuildTaskUpload, _super);
    function BuildTaskUpload() {
        _super.apply(this, arguments);
        this.requiredArguments = [argm.TASK_PATH];
        this.optionalArguments = [argm.OVERWRITE];
    }
    BuildTaskUpload.prototype.exec = function (args, options) {
        var _this = this;
        trace('build-task-upload.exec');
        var defer = Q.defer();
        this.checkArguments(args, options).then(function (allArguments) {
            var taskPath = allArguments[argm.TASK_PATH.name];
            var overwrite = allArguments[argm.OVERWRITE.name];
            trace('taskPath: ' + taskPath);
            try {
                vm.exists(taskPath, 'specified directory ' + taskPath + ' does not exist.');
            }
            catch (directoryError) {
                defer.reject(directoryError);
            }
            //directory is good, check json
            var tp = path.resolve(process.cwd(), path.join(taskPath, c_taskJsonFile));
            trace('task.json path: ' + tp);
            vm.validate(tp, 'no ' + c_taskJsonFile + ' in specified directory')
                .then(function (taskJson) {
                var archive = archiver('zip');
                archive.on('error', function (error) {
                    trace('Archiving error: ' + error.message);
                    error.message = 'Archiving error: ' + error.message;
                    defer.reject(error);
                });
                archive.directory(taskPath, false);
                trace("Initializing agent API...");
                var agentapi = _this.getWebApi().getTaskAgentApi(_this.connection.accountUrl);
                agentapi.uploadTaskDefinition(null, archive, taskJson.id, overwrite, function (err, statusCode, task) {
                    if (err) {
                        trace('TaskAgentApi.uploadTaskDefinition failed with code ' + statusCode + '. Message: ' + err.message);
                        err.statusCode = statusCode;
                        defer.reject(err);
                    }
                    else {
                        trace('Success');
                        defer.resolve({
                            sourceLocation: taskPath
                        });
                    }
                });
                archive.finalize();
            })
                .fail(function (error) {
                trace('Task json validation failed.');
                defer.reject(error);
            });
        })
            .fail(function (err) {
            trace('Failed to gather inputs. Message: ' + err.message);
            defer.reject(err);
        });
        return defer.promise;
    };
    BuildTaskUpload.prototype.output = function (data) {
        console.log('task at: ' + data.sourceLocation + ' uploaded successfully!');
    };
    return BuildTaskUpload;
})(cmdm.TfCommand);
exports.BuildTaskUpload = BuildTaskUpload;
