var inputs = require('./inputs');
var Q = require('q');
var csm = require('./credstore');
var argm = require('./arguments');
var trace = require('./trace');
var PatCredentials = (function () {
    function PatCredentials() {
        this.type = 'pat';
    }
    PatCredentials.prototype.fromString = function (creds) {
        // TODO: validate - it has a specific format
        this.token = creds.split(':')[1];
        this.populated = true;
        return this;
    };
    PatCredentials.prototype.toString = function () {
        return this.type + ':' + this.token;
    };
    PatCredentials.prototype.promptCredentials = function () {
        var _this = this;
        trace('PatCredentials.promptCredentials');
        var credInputs = [argm.PAT];
        return inputs.Qprompt(credInputs, []).then(function (result) {
            _this.token = result['token'];
            return _this;
        });
    };
    return PatCredentials;
})();
exports.PatCredentials = PatCredentials;
var BasicCredentials = (function () {
    function BasicCredentials() {
        this.type = 'basic';
    }
    BasicCredentials.prototype.fromString = function (creds) {
        var p = creds.split(':');
        this.username = p[1];
        this.password = p[2];
        this.populated = true;
        return this;
    };
    BasicCredentials.prototype.toString = function () {
        return this.type + ':' + this.username + ':' + this.password;
    };
    BasicCredentials.prototype.promptCredentials = function () {
        var _this = this;
        trace('BasicCredentials.promptCredentials');
        var credInputs = [argm.USERNAME, argm.PASSWORD];
        return inputs.Qprompt(credInputs, []).then(function (result) {
            _this.username = result['username'];
            _this.password = result['password'];
            return _this;
        });
    };
    return BasicCredentials;
})();
exports.BasicCredentials = BasicCredentials;
function getCredentials(url, authType) {
    trace('auth.getCredentials');
    var defer = Q.defer();
    // TODO: support other credential types
    var creds = null;
    return this.getCachedCredentials(url)
        .then(function (cachedData) {
        var type = cachedData.split(':')[0] || authType;
        switch (type) {
            case 'basic':
                creds = new BasicCredentials();
                break;
            case 'pat':
                creds = new PatCredentials();
                break;
            default:
                throw new Error('Unsupported auth type: ' + type);
        }
        return cachedData ? creds.fromString(cachedData) : creds;
    })
        .then(function (creds) {
        return creds.populated ? creds : creds.promptCredentials();
    });
    return defer.promise;
}
exports.getCredentials = getCredentials;
function getCachedCredentials(url) {
    trace('auth.getCachedCredentials');
    var defer = Q.defer();
    if (process.env['TFS_BYPASS_CACHE']) {
        trace('Skipping checking cache for credentials');
        defer.resolve('');
    }
    var cs = csm.getCredentialStore('tfx');
    cs.getCredential(url, 'allusers')
        .then(function (cred) {
        trace('Retrieved credentials from cache');
        defer.resolve(cred);
    })
        .fail(function (err) {
        trace('No credentials found in cache');
        defer.resolve('');
    });
    return defer.promise;
}
exports.getCachedCredentials = getCachedCredentials;
