/// <reference path="../../definitions/vso-node-api/vso-node-api.d.ts"/>
var inputs = require('./inputs');
var Q = require('q');
var url = require('url');
var apim = require('vso-node-api/WebApi');
var cm = require('./diskcache');
var argm = require('./arguments');
var trace = require('./trace');
var cache = new cm.DiskCache('tfx');
var TfsConnection = (function () {
    function TfsConnection(collectionUrl, credentials) {
        this.collectionUrl = collectionUrl;
        this.credentials = credentials;
        switch (credentials.type) {
            case "basic":
                trace('Using basic creds');
                var basicCreds = credentials;
                this.authHandler = apim.getBasicHandler(basicCreds.username, basicCreds.password);
                break;
            case "pat":
                trace('Using PAT creds');
                var patCreds = credentials;
                this.authHandler = apim.getBasicHandler("OAuth", patCreds.token);
                break;
        }
        var purl = url.parse(collectionUrl);
        if (!purl.protocol || !purl.host) {
            trace('Invalid collection url - protocol and host are required');
            throw new Error('Invalid collection url - protocol and host are required');
        }
        var splitPath = purl.path.split('/').slice(1);
        this.accountUrl = purl.protocol + '//' + purl.host;
        if (splitPath.length === 0 || (splitPath.length === 1 && splitPath[0] === '')) {
            trace('Invalid collection url - collection name is required. Eg: [accounturl]/[collectionname]');
            throw new Error('Invalid collection url - collection name is required. Eg: [accounturl]/[collectionname]');
        }
        if (splitPath.length === 2 && splitPath[0] === 'tfs') {
            //on prem
            this.accountUrl += '/' + 'tfs';
        }
        else if (splitPath.length > 1) {
            trace('Invalid collection url - path is too long. Collection url should take the form [accounturl]/[collectionname]');
            throw new Error('Invalid collection url - path is too long. Collection url should take the form [accounturl]/[collectionname]');
        }
    }
    return TfsConnection;
})();
exports.TfsConnection = TfsConnection;
var result;
function getCollectionUrl() {
    trace('loader.getCollectionUrl');
    var defer = Q.defer();
    return this.getCachedUrl()
        .then(function (url) {
        return url ? url : promptForUrl();
    });
    return defer.promise;
}
exports.getCollectionUrl = getCollectionUrl;
function getCachedUrl() {
    trace('loader.getCachedUrl');
    var defer = Q.defer();
    if (process.env['TFS_BYPASS_CACHE']) {
        trace('Skipping checking cache for collection url');
        defer.resolve('');
    }
    else {
        cache.getItem('cache', 'connection')
            .then(function (url) {
            trace('Retrieved collection url from cache');
            defer.resolve(url);
        })
            .fail(function (err) {
            trace('No collection url found in cache');
            defer.resolve('');
        });
    }
    return defer.promise;
}
exports.getCachedUrl = getCachedUrl;
function promptForUrl() {
    trace('loader.promptForUrl');
    var defer = Q.defer();
    var promise = defer.promise;
    if (result) {
        defer.resolve(result);
        return promise;
    }
    var credInputs = [argm.COLLECTION_URL];
    return inputs.Qprompt(credInputs, []).then(function (result) {
        return result[argm.COLLECTION_URL.name];
    });
}
exports.promptForUrl = promptForUrl;
