/// <reference path="../../definitions/vso-node-api/vso-node-api.d.ts"/>
var apim = require('vso-node-api/WebApi');
var inputm = require('./inputs');
var trace = require('./trace');
/*
 * formats lists of optional and required arguments for a command into a standard output for the command help
 * eg: 'args: <required1> <required2> [--optional1 <optional1>] [--optional2 <optional2>]'
 * @param requiredArguments an array of the names of all arguments that are required
 * @param requiredArguments an array of the names of all arguments that can be optionally provided
 * @param flags boolean flags (optional because less frequently used) eg "--all"
 */
function formatArgumentsHint(requiredArguments, optionalArguments, flags) {
    var argumentsHint = "";
    for (var i = 0; i < requiredArguments.length; i++) {
        argumentsHint += ' <' + requiredArguments[i].friendlyName + '>';
    }
    for (var i = 0; i < optionalArguments.length; i++) {
        argumentsHint += ' [--' + optionalArguments[i].name + ' <' + optionalArguments[i].friendlyName + '>]';
    }
    argumentsHint += ' [options]';
    return argumentsHint;
}
exports.formatArgumentsHint = formatArgumentsHint;
var TfCommand = (function () {
    function TfCommand() {
        this.requiredArguments = [];
        this.optionalArguments = [];
        this.flags = [];
    }
    // setConnection
    // getWebApi() 
    TfCommand.prototype.getWebApi = function () {
        return new apim.WebApi(this.connection.collectionUrl, this.connection.authHandler);
    };
    TfCommand.prototype.getArguments = function () {
        return formatArgumentsHint(this.requiredArguments, this.optionalArguments, this.flags);
    };
    //
    // should return a JSON data object which will be
    // - printed in json if --json, or
    // - passed back to output for readable text
    //
    TfCommand.prototype.exec = function (args, options) {
        var defer = Q.defer();
        trace('Unimplemented command');
        defer.reject(new Error('Not implemented.  Must override'));
        return defer.promise;
    };
    TfCommand.prototype.output = function (data) {
        // should override and output to console results
        // in readable text based on data from exec call
    };
    TfCommand.prototype.promptArguments = function (requiredInputs, optionalInputs) {
        trace('tfcommand.promptArguments');
        return inputm.Qprompt(requiredInputs, optionalInputs);
    };
    TfCommand.prototype.checkArguments = function (args, options) {
        trace('tfcommand.checkArguments');
        return inputm.Qcheck(args, options, this.requiredArguments, this.optionalArguments.concat(this.flags));
    };
    return TfCommand;
})();
exports.TfCommand = TfCommand;
